# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin
from django.utils.translation import ugettext as _

from .models import SessionInfo


# noinspection PyUnusedLocal
def invalidate_sessions(modeladmin, request, queryset):
    for session in queryset:
        session.erase()

invalidate_sessions.short_description = _("Log out users")


class SessionInfoAdmin(admin.ModelAdmin):
    list_display = ('user', 'creation_date', 'key', 'active', 'prefix', 'user_ip')
    list_filter = ('creation_date', 'prefix', 'active')
    search_fields = ['^user__username', '^user__email', '^user_ip']
    readonly_fields = ['user', 'creation_date', 'key', 'prefix', 'session_exists', 'session_exists_db', 'session_data']
    actions = [invalidate_sessions, ]

    def save_model(self, request, obj, form, change):
        if 'active' in form.changed_data and not obj.active:
            obj.erase()
        # model is read-only except of active

admin.site.register(SessionInfo, SessionInfoAdmin)



import uuid

from infosessions.session import SessionStore
from django.test import TestCase


class SessionTestCase(TestCase):
    def setUp(self):
        self.session = SessionStore()

    def test_modified(self):
        self.assertFalse(self.session.modified)
        self.session.create()
        self.session['kek'] = 'pek'
        self.assertTrue(self.session.modified)
        self.assertIsNotNone(self.session.session_key)


class SessionPersistenceTestCase(TestCase):
    def setUp(self):
        self.key = 'key'
        self.value = str(uuid.uuid4())
        session = SessionStore()
        session[self.key] = self.value
        session.save()
        self.session_key = session.session_key

    def test_key_exist(self):
        s = SessionStore()
        self.assertTrue(s.exists(self.session_key))

    def test_delete(self):
        s = SessionStore(self.session_key)
        s.delete()
        self.assertFalse(s.exists(self.session_key))

    def test_value_changed(self):
        session = SessionStore(self.session_key)
        self.assertEqual(session[self.key], self.value)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import logging

from django.contrib.auth import logout

from .models import SessionInfo
from .constants import SESSION_PROCESSED

logger = logging.getLogger(__name__)


def is_logged(request):
    return hasattr(request, 'user') and request.user.is_authenticated()


def is_processed(request):
    return SESSION_PROCESSED in request.META


class SessionSyncMiddleware(object):

    @staticmethod
    def process_request(request):
        logger.info('processing request')
        if is_processed(request):
            return
        if is_logged(request):
            if not SessionInfo.exists(request):
                SessionInfo.record(request)
        request.META[SESSION_PROCESSED] = True

    @classmethod
    def process_response(cls, request, response):
        if not is_processed(request):
            cls.process_request(request)
        return response


class SessionSameIP(object):

    @staticmethod
    def process_request(request):
        if not is_processed(request):
            return
        if is_logged(request) and not SessionInfo.is_request_valid(request):
            session_key = request.session.session_key
            logger.warning('logging out user with session key %s; ip changed' % session_key)
            logout(request)
            request.session.delete()
